@@ -29,6 +29,7 @@ class AppManagement:
         self.modules = {}
         self.objects = {}
         self.check_app_updates_profile_stats = None
+        self.check_updates_lock = None
 
         # Initialize config file tracking
 
@@ -102,6 +103,9 @@ class AppManagement:
         await self.AD.state.remove_entity("admin", "app.{}".format(name))
 
     async def init_admin_stats(self):
+        # store lock
+        self.check_updates_lock = asyncio.Lock()
+
         # create sensors
         await self.add_entity(self.active_apps_sensor, 0, {"friendly_name": "Active Apps"})
         await self.add_entity(self.inactive_apps_sensor, 0, {"friendly_name": "Inactive Apps"})
@@ -231,6 +235,8 @@ class AppManagement:
 
         self.AD.futures.cancel_futures(name)
 
+        self.AD.services.clear_services(name)
+
         await self.AD.sched.terminate_app(name)
 
         await self.set_state(name, state="terminated")
@@ -357,6 +363,29 @@ class AppManagement:
             "running": False,
         }
 
+    def init_sequence_object(self, name, object):
+        """Initialize the sequence"""
+
+        self.objects[name] = {
+            "type": "sequence",
+            "object": object,
+            "id": uuid.uuid4().hex,
+            "pin_app": False,
+            "pin_thread": -1,
+            "running": False,
+        }
+
+    async def terminate_sequence(self, name: str) -> bool:
+        """Terminate the sequence"""
+
+        if name in self.objects:
+            del self.objects[name]
+
+        await self.AD.callbacks.clear_callbacks(name)
+        self.AD.futures.cancel_futures(name)
+
+        return True
+
     async def read_config(self):  # noqa: C901
 
         new_config = None
@@ -367,8 +396,8 @@ class AppManagement:
             )
             new_config = utils.run_in_executor(self.read_config_file, self.app_config_file)
         else:
-            for root, subdirs, files in os.walk(self.AD.app_dir):
-                subdirs[:] = [d for d in subdirs if d not in self.AD.exclude_dirs]
+            for root, subdirs, files in await utils.run_in_executor(self, os.walk, self.AD.app_dir):
+                subdirs[:] = [d for d in subdirs if d not in self.AD.exclude_dirs and "." not in d]
                 if root[-11:] != "__pycache__" and root[0] != ".":
                     for file in files:
                         if file[-5:] == ".yaml" and file[0] != ".":
@@ -501,7 +530,7 @@ class AppManagement:
             later_files["latest"] = last_latest
             later_files["deleted"] = []
             for root, subdirs, files in os.walk(self.AD.app_dir):
-                subdirs[:] = [d for d in subdirs if d not in self.AD.exclude_dirs]
+                subdirs[:] = [d for d in subdirs if d not in self.AD.exclude_dirs and "." not in d]
                 if root[-11:] != "__pycache__" and root[0] != ".":
                     for file in files:
                         if file[-5:] == ".yaml" and file[0] != ".":
@@ -788,7 +817,7 @@ class AppManagement:
                     #
                     # Prune dir list
                     #
-                    subdirs[:] = [d for d in subdirs if d not in self.AD.exclude_dirs]
+                    subdirs[:] = [d for d in subdirs if d not in self.AD.exclude_dirs and "." not in d]
 
                     ext = filter["input_ext"]
                     extlen = len(ext) * -1
@@ -836,204 +865,205 @@ class AppManagement:
         fh.close()
 
     # @_timeit
-    async def check_app_updates(self, plugin=None, mode="normal"):  # noqa: C901
+    async def check_app_updates(self, plugin: str = None, mode: str = "normal"):  # noqa: C901
 
-        if self.AD.apps is False:
-            return
+        async with self.check_updates_lock:
+            if self.AD.apps is False:
+                return
 
-        # Lets add some profiling
-        pr = None
-        if self.AD.check_app_updates_profile is True:
-            pr = cProfile.Profile()
-            pr.enable()
+            # Lets add some profiling
+            pr = None
+            if self.AD.check_app_updates_profile is True:
+                pr = cProfile.Profile()
+                pr.enable()
 
-        # Process filters
+            # Process filters
 
-        await utils.run_in_executor(self, self.process_filters)
+            await utils.run_in_executor(self, self.process_filters)
 
-        # Get list of apps we need to terminate and/or initialize
+            # Get list of apps we need to terminate and/or initialize
 
-        apps = await self.check_config()
+            apps = await self.check_config()
 
-        found_files = []
-        modules = []
-        for root, subdirs, files in await utils.run_in_executor(self, os.walk, self.AD.app_dir, topdown=True):
-            # print(root, subdirs, files)
-            #
-            # Prune dir list
-            #
-            subdirs[:] = [d for d in subdirs if d not in self.AD.exclude_dirs]
+            found_files = []
+            modules = []
+            for root, subdirs, files in await utils.run_in_executor(self, os.walk, self.AD.app_dir, topdown=True):
+                # print(root, subdirs, files)
+                #
+                # Prune dir list
+                #
+                subdirs[:] = [d for d in subdirs if d not in self.AD.exclude_dirs and "." not in d]
 
-            if root[-11:] != "__pycache__" and root[0] != ".":
-                if root not in self.module_dirs:
-                    self.logger.info("Adding %s to module import path", root)
-                    sys.path.insert(0, root)
-                    self.module_dirs.append(root)
+                if root[-11:] != "__pycache__" and root[0] != ".":
+                    if root not in self.module_dirs:
+                        self.logger.info("Adding %s to module import path", root)
+                        sys.path.insert(0, root)
+                        self.module_dirs.append(root)
 
-            for file in files:
-                if file[-3:] == ".py" and file[0] != ".":
-                    found_files.append(os.path.join(root, file))
+                for file in files:
+                    if file[-3:] == ".py" and file[0] != ".":
+                        found_files.append(os.path.join(root, file))
 
-        for file in found_files:
-            if file == os.path.join(self.AD.app_dir, "__init__.py"):
-                continue
-            try:
-
-                # check we can actually open the file
-                await utils.run_in_executor(self, self.check_file, file)
-
-                modified = await utils.run_in_executor(self, os.path.getmtime, file)
-                if file in self.monitored_files:
-                    if self.monitored_files[file] < modified:
-                        modules.append({"name": file, "reload": True})
-                        self.monitored_files[file] = modified
-                else:
-                    self.logger.debug("Found module %s", file)
-                    modules.append({"name": file, "reload": False})
-                    self.monitored_files[file] = modified
-            except IOError as err:
-                self.logger.warning("Unable to read app %s: %s - skipping", file, err)
-
-        # Check for deleted modules and add them to the terminate list
-        deleted_modules = []
-        for file in self.monitored_files:
-            if file not in found_files or mode == "term":
-                deleted_modules.append(file)
-                self.logger.info("Removing module %s", file)
-
-        for file in deleted_modules:
-            del self.monitored_files[file]
-            for app in self.apps_per_module(self.get_module_from_path(file)):
-                apps["term"][app] = 1
-
-        # Add any apps we need to reload because of file changes
-
-        for module in modules:
-            for app in self.apps_per_module(self.get_module_from_path(module["name"])):
-                if module["reload"]:
-                    apps["term"][app] = 1
-                apps["init"][app] = 1
-
-            if "global_modules" in self.app_config:
-                for gm in utils.single_or_list(self.app_config["global_modules"]):
-                    if gm == self.get_module_from_path(module["name"]):
-                        for app in self.apps_per_global_module(gm):
-                            if module["reload"]:
-                                apps["term"][app] = 1
-                            apps["init"][app] = 1
-
-        if plugin is not None:
-            self.logger.info("Processing restart for %s", plugin)
-            # This is a restart of one of the plugins so check which apps need to be restarted
-            for app in self.app_config:
-                reload = False
-                if app in self.non_apps:
+            for file in found_files:
+                if file == os.path.join(self.AD.app_dir, "__init__.py"):
                     continue
-                if "plugin" in self.app_config[app]:
-                    for this_plugin in utils.single_or_list(self.app_config[app]["plugin"]):
-                        if this_plugin == plugin:
-                            # We got a match so do the reload
-                            reload = True
-                            break
-                        elif plugin == "__ALL__":
-                            reload = True
-                            break
-                else:
-                    # No plugin dependency specified, reload to error on the side of caution
-                    reload = True
+                try:
 
-                if reload is True:
+                    # check we can actually open the file
+                    await utils.run_in_executor(self, self.check_file, file)
+
+                    modified = await utils.run_in_executor(self, os.path.getmtime, file)
+                    if file in self.monitored_files:
+                        if self.monitored_files[file] < modified:
+                            modules.append({"name": file, "reload": True})
+                            self.monitored_files[file] = modified
+                    else:
+                        self.logger.debug("Found module %s", file)
+                        modules.append({"name": file, "reload": False})
+                        self.monitored_files[file] = modified
+                except IOError as err:
+                    self.logger.warning("Unable to read app %s: %s - skipping", file, err)
+
+            # Check for deleted modules and add them to the terminate list
+            deleted_modules = []
+            for file in self.monitored_files:
+                if file not in found_files or mode == "term":
+                    deleted_modules.append(file)
+                    self.logger.info("Removing module %s", file)
+
+            for file in deleted_modules:
+                del self.monitored_files[file]
+                for app in self.apps_per_module(self.get_module_from_path(file)):
                     apps["term"][app] = 1
+
+            # Add any apps we need to reload because of file changes
+
+            for module in modules:
+                for app in self.apps_per_module(self.get_module_from_path(module["name"])):
+                    if module["reload"]:
+                        apps["term"][app] = 1
                     apps["init"][app] = 1
 
-        # Terminate apps
+                if "global_modules" in self.app_config:
+                    for gm in utils.single_or_list(self.app_config["global_modules"]):
+                        if gm == self.get_module_from_path(module["name"]):
+                            for app in self.apps_per_global_module(gm):
+                                if module["reload"]:
+                                    apps["term"][app] = 1
+                                apps["init"][app] = 1
 
-        apps_terminated = {}  # store apps properly terminated is any
-        if apps is not None and apps["term"]:
-            prio_apps = self.get_app_deps_and_prios(apps["term"], mode)
-
-            for app in sorted(prio_apps, key=prio_apps.get, reverse=True):
-                executed = await self.stop_app(app)
-                apps_terminated[app] = executed
-
-        # Load/reload modules
-
-        for mod in modules:
-            try:
-                await utils.run_in_executor(self, self.read_app, mod["name"], mod["reload"])
-            except Exception:
-                self.error.warning("-" * 60)
-                self.error.warning("Unexpected error loading module: %s:", mod["name"])
-                self.error.warning("-" * 60)
-                self.error.warning(traceback.format_exc())
-                self.error.warning("-" * 60)
-                if self.AD.logging.separate_error_log() is True:
-                    self.logger.warning("Unexpected error loading module: %s:", mod["name"])
-
-                self.logger.warning("Removing associated apps:")
-                module = self.get_module_from_path(mod["name"])
+            if plugin is not None:
+                self.logger.info("Processing restart for %s", plugin)
+                # This is a restart of one of the plugins so check which apps need to be restarted
                 for app in self.app_config:
-                    if "module" in self.app_config[app] and self.app_config[app]["module"] == module:
-                        if apps["init"] and app in apps["init"]:
-                            del apps["init"][app]
-                            self.logger.warning("%s", app)
-                            await self.set_state(app, state="compile_error")
+                    reload = False
+                    if app in self.non_apps:
+                        continue
+                    if "plugin" in self.app_config[app]:
+                        for this_plugin in utils.single_or_list(self.app_config[app]["plugin"]):
+                            if this_plugin == plugin:
+                                # We got a match so do the reload
+                                reload = True
+                                break
+                            elif plugin == "__ALL__":
+                                reload = True
+                                break
+                    else:
+                        # No plugin dependency specified, reload to error on the side of caution
+                        reload = True
 
-        if apps is not None and apps["init"]:
+                    if reload is True:
+                        apps["term"][app] = 1
+                        apps["init"][app] = 1
 
-            prio_apps = self.get_app_deps_and_prios(apps["init"], mode)
+            # Terminate apps
 
-            # Load Apps
+            apps_terminated = {}  # store apps properly terminated is any
+            if apps is not None and apps["term"]:
+                prio_apps = self.get_app_deps_and_prios(apps["term"], mode)
 
-            for app in sorted(prio_apps, key=prio_apps.get):
+                for app in sorted(prio_apps, key=prio_apps.get, reverse=True):
+                    executed = await self.stop_app(app)
+                    apps_terminated[app] = executed
+
+            # Load/reload modules
+
+            for mod in modules:
                 try:
+                    await utils.run_in_executor(self, self.read_app, mod["name"], mod["reload"])
+                except Exception:
+                    self.error.warning("-" * 60)
+                    self.error.warning("Unexpected error loading module: %s:", mod["name"])
+                    self.error.warning("-" * 60)
+                    self.error.warning(traceback.format_exc())
+                    self.error.warning("-" * 60)
+                    if self.AD.logging.separate_error_log() is True:
+                        self.logger.warning("Unexpected error loading module: %s:", mod["name"])
+
+                    self.logger.warning("Removing associated apps:")
+                    module = self.get_module_from_path(mod["name"])
+                    for app in self.app_config:
+                        if "module" in self.app_config[app] and self.app_config[app]["module"] == module:
+                            if apps["init"] and app in apps["init"]:
+                                del apps["init"][app]
+                                self.logger.warning("%s", app)
+                                await self.set_state(app, state="compile_error")
+
+            if apps is not None and apps["init"]:
+
+                prio_apps = self.get_app_deps_and_prios(apps["init"], mode)
+
+                # Load Apps
+
+                for app in sorted(prio_apps, key=prio_apps.get):
+                    try:
+                        if "disable" in self.app_config[app] and self.app_config[app]["disable"] is True:
+                            self.logger.info("%s is disabled", app)
+                            await self.set_state(app, state="disabled")
+                            await self.increase_inactive_apps(app)
+                        else:
+                            if apps_terminated.get(app, True) is True:  # the app terminated properly
+                                await self.init_object(app)
+
+                            else:
+                                self.logger.warning("Cannot initialize app %s, as it didn't terminate properly", app)
+
+                    except Exception:
+                        error_logger = logging.getLogger("Error.{}".format(app))
+                        error_logger.warning("-" * 60)
+                        error_logger.warning("Unexpected error initializing app: %s:", app)
+                        error_logger.warning("-" * 60)
+                        error_logger.warning(traceback.format_exc())
+                        error_logger.warning("-" * 60)
+                        if self.AD.logging.separate_error_log() is True:
+                            self.logger.warning(
+                                "Logged an error to %s", self.AD.logging.get_filename("error_log"),
+                            )
+
+                await self.AD.threading.calculate_pin_threads()
+
+                # Call initialize() for apps
+
+                for app in sorted(prio_apps, key=prio_apps.get):
                     if "disable" in self.app_config[app] and self.app_config[app]["disable"] is True:
-                        self.logger.info("%s is disabled", app)
-                        await self.set_state(app, state="disabled")
-                        await self.increase_inactive_apps(app)
+                        pass
                     else:
                         if apps_terminated.get(app, True) is True:  # the app terminated properly
-                            await self.init_object(app)
+                            await self.initialize_app(app)
 
                         else:
-                            self.logger.warning("Cannot initialize app %s, as it didn't terminate properly", app)
+                            self.logger.debug("Cannot initialize app %s, as it didn't terminate properly", app)
 
-                except Exception:
-                    error_logger = logging.getLogger("Error.{}".format(app))
-                    error_logger.warning("-" * 60)
-                    error_logger.warning("Unexpected error initializing app: %s:", app)
-                    error_logger.warning("-" * 60)
-                    error_logger.warning(traceback.format_exc())
-                    error_logger.warning("-" * 60)
-                    if self.AD.logging.separate_error_log() is True:
-                        self.logger.warning(
-                            "Logged an error to %s", self.AD.logging.get_filename("error_log"),
-                        )
+            if self.AD.check_app_updates_profile is True:
+                pr.disable()
 
-            await self.AD.threading.calculate_pin_threads()
+            s = io.StringIO()
+            sortby = "cumulative"
+            ps = pstats.Stats(pr, stream=s).sort_stats(sortby)
+            ps.print_stats()
+            self.check_app_updates_profile_stats = s.getvalue()
 
-            # Call initialize() for apps
-
-            for app in sorted(prio_apps, key=prio_apps.get):
-                if "disable" in self.app_config[app] and self.app_config[app]["disable"] is True:
-                    pass
-                else:
-                    if apps_terminated.get(app, True) is True:  # the app terminated properly
-                        await self.initialize_app(app)
-
-                    else:
-                        self.logger.debug("Cannot initialize app %s, as it didn't terminate properly", app)
-
-        if self.AD.check_app_updates_profile is True:
-            pr.disable()
-
-        s = io.StringIO()
-        sortby = "cumulative"
-        ps = pstats.Stats(pr, stream=s).sort_stats(sortby)
-        ps.print_stats()
-        self.check_app_updates_profile_stats = s.getvalue()
-
-        self.apps_initialized = True
+            self.apps_initialized = True
 
     def get_app_deps_and_prios(self, applist, mode):
 
