@@ -93,10 +93,14 @@ class State:
         """Used to add a database file for a created namespace"""
 
         try:
+            if namespace in self.state and isinstance(self.state[namespace], utils.PersistentDict):
+                self.logger.info("Persistent Namespace '%s' already initialized", namespace)
+                return
+
             nspath = os.path.join(self.AD.config_dir, "namespaces")
             safe = bool(writeback == "safe")
-
             nspath_file = os.path.join(nspath, f"{namespace}.db")
+
             self.state[namespace] = utils.PersistentDict(nspath_file, safe)
 
             self.logger.info("Persistent Namespace '%s' initialized", namespace)
@@ -188,7 +192,8 @@ class State:
             # If we have a timeout parameter, add a scheduler entry to delete the callback later
             #
             if "timeout" in kwargs:
-                exec_time = await self.AD.sched.get_now() + datetime.timedelta(seconds=int(kwargs["timeout"]))
+                timeout = kwargs.pop("timeout")
+                exec_time = await self.AD.sched.get_now() + datetime.timedelta(seconds=int(timeout))
 
                 kwargs["__timeout"] = await self.AD.sched.insert_schedule(
                     name, exec_time, None, False, None, __state_handle=handle,
@@ -209,12 +214,11 @@ class State:
                     if "attribute" in kwargs:
                         __attribute = kwargs["attribute"]
                     if "new" in kwargs:
-                        if __attribute is None and self.state[namespace][entity]["state"] == kwargs["new"]:
+                        if __attribute is None and self.state[namespace][entity].get("state") == kwargs["new"]:
                             __new_state = kwargs["new"]
                         elif (
                             __attribute is not None
-                            and __attribute in self.state[namespace][entity]["attributes"]
-                            and self.state[namespace][entity]["attributes"][__attribute] == kwargs["new"]
+                            and self.state[namespace][entity]["attributes"].get(__attribute) == kwargs["new"]
                         ):
                             __new_state = kwargs["new"]
                         else:
@@ -615,7 +619,10 @@ class State:
         if hasattr(plugin, "set_plugin_state"):
             # We assume that the state change will come back to us via the plugin
             self.logger.debug("sending event to plugin")
-            result = await plugin.set_plugin_state(namespace, entity, **kwargs)
+
+            result = await plugin.set_plugin_state(
+                namespace, entity, state=new_state["state"], attributes=new_state["attributes"]
+            )
             if result is not None:
                 if "entity_id" in result:
                     result.pop("entity_id")
