@@ -134,7 +134,26 @@ function basejavascript(widget_id, url, skin, parameters)
                 i++
             }
         }
-
+        else
+        {
+            if ("timeout" in parameters)
+            {
+                current_dash = location.pathname.split('/')[1];
+		if (current_dash.length > 0)
+		{
+                    if (i == 0)
+                    {
+                        url = url + "?return=" + current_dash;
+                        i++
+                    }
+                    else
+                    {
+                        url = url + "&return=" + current_dash;
+                        i++
+                     }
+                }
+            }
+        }
         if ("timeout" in parameters)
         {
             if (i == 0)
